"""
https://docs.python.org/zh-cn/3/library/unittest.html
python -m unittest test_module1 test_module2
python -m unittest test_module.TestClass
python -m unittest test_module.TestClass.test_method
python -m unittest tests/test_something.py
"""

import unittest
 
def test_example ():
    pass
 
if __name__ == '__main__': 
    unittest.main()